#ifndef __adv_words_H
#define __adv_words_H

#include "aconst.h"
/****************************************************
 *
 *	 Defines structs to hold verbs, nouns and
 *	 special data related to some objects.
 *
 ****************************************************/

//Declares current item.
extern unsigned char Itm1;
#pragma zpsym ("Itm1")

//Holds each verb entry.
typedef struct verb_
{
	//Verb name.
	unsigned char* Name;
	//Routine to perform action.
	void (*vdo) (void);
};

//Declares verbs.
//NumVerbs is defined in aconst.h.  If you need more
//verbs, increase this number and add the verbs to the "wordsdb.c"
//file.
extern struct verb_ Verb [NumVerbs];

//Routine to return the current input's verb #
//or 0xFF if not found.
unsigned char FindVerb (void);

//Holds an item's data.
typedef struct item_
{
	//Name of item.
	unsigned char* Name;
	//Bits containing item's attributes, including
	//if item is movable or if its state is changeable.
	unsigned char Attrib;
	//Help displayed if user types "look <item>."
	unsigned char *Info;
	//Pad to make struct size 8 for easier calculations.
	unsigned char x;
};

//Declares items and a pointer to the current item.
extern struct item_ Item [NumItems], *ItemPtr;
#pragma zpsym ("ItemPtr")

//Declares entry for aliases, a.k.a. synonyms.
typedef struct item_Alias
{
	//Name of alias.
	unsigned char* Name;
	//Number of aliased item.
	unsigned char Num;
	//Padding to make struct 4 bytes.
	unsigned char x;
};
//Declares aliases.
extern struct item_Alias ItemAlias [NumItemAliases];

//Declares category entry, where a word can mean any of several other words.
typedef struct ItemCate
{
	//Category name.
	char* Name;
	//Possible meanings.  End lists with -1.
	char Items[6];
};
extern struct ItemCate ItemCate [NumItemCate];

unsigned char FindItem (void);

/* item state: */
#define itemSOpen	0x0001
	/*True if open, false if closed.*/
#define itemSOn 	0x0001
	/*True if on, false if off.*/
#define itemSLocked	0x0002
	/*True if locked.*/
#define itemSSpecState	0xFF00
	/*Dependent on item.*/
#define itemSRead	0x0004

/* item attributes: */
#define itemAMovable	0x01
	/*If true, can be picked up, moved, given, etc.*/
#define itemAStateChangable 0x02
	/*If true, can be open, closed and changed.*/

unsigned char __fastcall__ iSearch (unsigned char* i);

/****************************************************
   Item Numbers
 ****************************************************/

//Enum for all known items:
#define iKey		  0

/* Partial Items/Categories */
#define iMap_		128

//Directions.
#define iNorth		160
#define iSouth		161
#define iEast		162
#define iWest		163

/* People srart at 192 */

/****************************************************
   General verb-handlers
 ****************************************************/

void vLook	(void);
void vGo	(void);
void vItems	(void);
void vBuy	(void);

void vTeleport	(void);
void vExit	(void);
void vEnter	(void);

void vSay	(void);
void vAnswer	(void);


void vGet	(void);
void vPut	(void);
void vRead	(void);


void vIntro	(void);
void vHelp	(void);

void vGoNorth	(void);
void vGoSouth	(void);
void vGoEast	(void);
void vGoWest	(void);



/****************************************************
   Message Numbers
 ****************************************************/
 //Declares the text for some often-used messages.
extern const char* Message [];
//Enum of such messages.
#define msgDontHave 		  0
#define msgHoldingTooManyItems	  1
#define msgTooManyItemsInRoom	  2
#define msgAlreadyHave		  3
#define msgCantGoThere		  4
#define msgItemNotAvail		  5
#define msgBeMoreSpec		  6

//Gets a line from the user.
//In points to the buffer to which the input is written.
//InLen is the length of the buffer without the terminating NUL.
//Returns th size of actual input without the terminating NUL.
unsigned char __fastcall__ GetInput (char* In, unsigned char InLen);

#endif

